<?php


namespace AcyChecker\Controllers;


use AcyChecker\Classes\TestClass;
use AcyChecker\Libraries\AcycController;
use AcyChecker\Services\PaginationService;
use AcyCheckerCmsServices\Database;
use AcyChecker\Services\HttpService;
use AcyCheckerCmsServices\Form;
use AcyCheckerCmsServices\Language;
use AcyCheckerCmsServices\Message;
use AcyCheckerCmsServices\Router;
use AcyCheckerCmsServices\Security;
use AcyCheckerCmsServices\Url;

class TestsController extends AcycController
{
    const PAGINATION_NAME = 'test_pagination';
    const DEFAULT_ELEMENTS_PER_PAGE = 20;

    private $testClass;

    public function __construct()
    {
        parent::__construct();

        $this->name = 'Tests';
        $this->defaultTask = 'listing';

        $this->testClass = new TestClass();
    }

    private function getNumberOfElementsToDisplay()
    {
        $selectValue = Security::getVar('int', 'acyc_element_per_page', $this->config->get('listing_element_page', self::DEFAULT_ELEMENTS_PER_PAGE));

        $this->config->save(['listing_element_page' => $selectValue]);

        return $selectValue;
    }

    public function listing()
    {
        $this->layout = 'listing';

        $data = [
            'per_page' => $this->getNumberOfElementsToDisplay(),
            'search' => $this->getVarFiltersListing('string', 'search', ''),
            'page' => $this->getVarFiltersListing('int', self::PAGINATION_NAME, 1),
            'current_status' => $this->getVarFiltersListing('string', 'status', ''),
        ];

        $this->prepareTestsElements($data);
        $this->preparePagination($data);
        $this->prepareTexts($data);
        $this->prepareStatus($data);
        $this->displayInfoInProgress();

        $this->breadcrumb[Language::translation('ACYC_TESTS')] = Url::completeLink('tests');

        $this->display($data);
    }

    private function prepareTestsElements(&$data)
    {
        $data['offset'] = ($data['page'] - 1) * $data['per_page'];
        $data = array_merge($data, $this->testClass->getMatchingElements($data));
    }

    private function preparePagination(&$data)
    {
        $data['pagination'] = new PaginationService($data['page'], $data['total'], $data['per_page'], self::PAGINATION_NAME);
    }

    private function prepareTexts(&$data)
    {
        $data['final_status_texts'] = [
            'not_existing' => Language::translation('ACYC_NOT_EXISTING'),
            'risky' => Language::translation('ACYC_RISKY'),
            'possible' => Language::translation('ACYC_POSSIBLE'),
            'valid' => Language::translation('ACYC_VALID'),
        ];
    }

    private function prepareStatus(&$data)
    {
        $data['status'] = [
            'all' => [
                'text' => Language::translation('ACYC_ALL'),
                'number' => $this->testClass->getStatusNumber('all', $data),
                'value' => 'all',
            ],
            'tested' => [
                'text' => Language::translation('ACYC_TESTED'),
                'number' => $this->testClass->getStatusNumber('tested', $data),
                'value' => 'tested',
            ],
            'progress' => [
                'text' => Language::translation('ACYC_IN_PROGRESS'),
                'number' => $this->testClass->getStatusNumber('progress', $data),
                'value' => 'progress',
            ],
        ];
    }

    public function displayInfoInProgress()
    {
        $tests = $this->config->get('urls_results_batch');
        $tests = json_decode($tests, true);
        if (!empty($tests)) {
            $nbBatchs = count($tests);
            Message::enqueueMessage(Language::translationSprintf('ACYC_TEST_IN_PROGRESS_X_BATCH', $nbBatchs), 'info');
        }
    }

    public function doexport()
    {
        $separator = '","';
        $eol = "\r\n";
        $before = '"';
        $after = '"';

        error_reporting(E_ALL);
        @ini_set('display_errors', 1);

        $columns = Database::loadResultArray('SHOW COLUMNS FROM #__acyc_test');

        $query = 'SELECT * FROM #__acyc_test AS test';
        $elements = Database::loadObjectList($query);

        if (empty($elements)) {
            // Remove noheader in url
            $completLink = rtrim(Url::completeLink('tests', false, true), '&noheader=1');
            Message::enqueueMessage(Language::translation('ACYC_NO_DATA_TO_EXPORT'), 'warning');
            Router::redirect($completLink);
        }

        $lines = [];
        $lines[] = $before.implode($separator, $columns).$after;

        foreach ($elements as $oneElement) {
            $line = [];
            $oneElement->status = $oneElement->status ? Language::translation('ACYC_COMPLETED') : Language::translation('ACYC_IN_PROGRESS');
            foreach ($oneElement as $col => $item) {
                if ($col == 'raw_result') {
                    $array = json_decode($item);
                    $suggestions = $array->domain_suggestions;
                    $line[] = htmlspecialchars(implode(',', $suggestions), ENT_QUOTES, 'UTF-8');
                } else {
                    $line[] = htmlspecialchars($item, ENT_QUOTES, 'UTF-8');
                }
            }
            $lines[] = $before.implode($separator, $line).$after;
        }
        $final = implode($eol, $lines);

        @ob_get_clean();
        $filename = 'export_acychecker_'.date('Y-m-d');
        HttpService::setDownloadHeaders($filename);

        echo $final;
        exit;
    }


    public function clearTests()
    {
        $testClass = new TestClass();

        $testClass->deleteAllTests();

        $this->listing();
    }
}
